#!/usr/bin/env python

# Script that can decode a raw haret log of traces into a list of
# read/written serial strings.

import sys
import re
import os
import tempfile
import struct

import dis

HEXALL=0

redebug = dis.redebug
reirq = dis.reirq

# Codes to watch for
SCANTRACE = 1
WRITEINSN = (0xe5839010, 0xe5823010)
READINSN = (0xe5923010, 0xe5921010)
##SCANTRACE = 0
##WRITEINSN = 0x0935acb0
##READINSN = 0x0935a9f0

# Buffer variables
thisRead = lastWrite = None

def procline(line):
    global thisRead, lastWrite
    if SCANTRACE:
        # Look at memory trace events
        m = redebug.match(line)
        if m is None:
            return dis.procline(line)
        insn = int(m.group('insn'), 16)
        Rd = int(m.group('Rd'), 16)
    else:
        # Look at breakpoints
        m = reirq.match(line)
        if m is None:
            return dis.procline(line)
        parts = line.split()
        if parts[2] != 'insn':
            return dis.procline(line)
        insn = int(parts[3][:-1], 16)
        if insn not in WRITEINSN + READINSN:
            return
        Rd = int(parts[4], 16)
##     print "--",
##     dis.procline(line)
    if insn in WRITEINSN:
        if Rd in [0xcff2, 0x1002]:
            # Ignore
            return
        if Rd == 0:
            if lastWrite is None:
                print "Got zero write without a previous write"
            thisRead = lastWrite
            lastWrite = None
            return
        lastWrite = Rd
    elif insn in READINSN:
        if thisRead is None:
            return
        regname = {0x104c: 'AUX1', 0x1050: 'AUX2', 0x1054: 'AUX3'
                   , 0x1058: 'BAT1', 0x105c: 'BAT2'}.get(thisRead, '??')
        thisRead = None
        print "%s: %s=%d" % (dis.getClock(m), regname, Rd)
    else:
        return dis.procline(line)

def main():
    lines = sys.stdin.readlines()
    for line in lines:
        procline(line)

if __name__ == '__main__':
    main()
